package com.fjhx.sale.service.contract.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.controller.utils.DateUtils;
import com.fjhx.account.entity.account.po.AccountManagement;
import com.fjhx.account.service.account.AccountManagementService;
import com.fjhx.area.service.SetCustomizeAreaId;
import com.fjhx.area.utils.CustomizeAreaUtil;
import com.fjhx.common.entity.corporation.po.Corporation;
import com.fjhx.common.entity.documentary.bo.DocumentaryData;
import com.fjhx.common.enums.FlowStatusEnum;
import com.fjhx.common.service.corporation.CorporationService;
import com.fjhx.common.service.documentary.GetDocumentaryBusinessTemplate;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.item.entity.product.dto.ProductInfoSelectDto;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.contract.bo.ContractDocumentaryBo;
import com.fjhx.sale.entity.contract.dto.ContractDto;
import com.fjhx.sale.entity.contract.dto.ContractSelectDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.vo.ContractDocumentaryVo;
import com.fjhx.sale.entity.contract.vo.ContractPdfInfoVo;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.sale.vo.SaleQuotationVo;
import com.fjhx.sale.mapper.contract.ContractMapper;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.ContractService;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.framework.config.ThreadPoolConfig;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;


/**
 * <p>
 * 外销合同表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-17
 */
@Service
public class ContractServiceImpl extends ServiceImpl<ContractMapper, Contract>
        implements ContractService, GetDocumentaryBusinessTemplate {

    private static final SimpleDateFormat sdf = new SimpleDateFormat("dd/MMM/yyyy", Locale.ENGLISH);

    @Qualifier(ThreadPoolConfig.threadPoolTaskExecutor)
    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Autowired
    private CustomerService customerService;

    @Autowired
    private CorporationService corporationService;

    @Autowired
    private ContractProductService contractProductService;

    @Autowired
    private ProductInfoService productInfoService;

    @Autowired
    private AccountManagementService accountManagementService;

    /**
     * 分页
     *
     * @param dto
     * @return
     */
    @Override
    public Page<ContractVo> getPage(ContractSelectDto dto) {
        IWrapper<Contract> wrapper = getWrapper();
        wrapper.orderByDesc("t1", Contract::getCreateTime);
        wrapper.between("t1", Contract::getStatus, FlowStatusEnum.DRAFT.getKey(), FlowStatusEnum.CANCELLATION.getKey() - 1);
        if (StringUtils.isNotEmpty(dto.getStatus())) {
            wrapper.eq("t1", Contract::getStatus, dto.getStatus());
        }
        if (StringUtils.isNotEmpty(dto.getSellCorporationId())) {
            wrapper.eq("t1", Contract::getSellCorporationId, dto.getSellCorporationId());
        }
        if (StringUtils.isNotEmpty(dto.getKeyword())) {
            wrapper.keyword(dto.getKeyword(), new SqlField("t1", Contract::getCode));
        }
        Page<ContractVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ContractVo> list = page.getRecords();
        if(CollectionUtils.isNotEmpty(list)){
            List<Long> customerIds = list.stream().map(Contract::getBuyCorporationId).collect(Collectors.toList());
            List<Customer> customerList = customerService.list(Wrappers.<Customer>query().lambda().in(Customer::getId, customerIds));
            Map<Long, List<Customer>> cusMap = customerList.stream().distinct().collect(Collectors.groupingBy(Customer::getId));

            List<Long> ids = list.stream().map(ContractVo::getId).collect(Collectors.toList());
            //获取交接单附件
            Map<Long, List<FileInfoVo>> fileMap = ObsFileUtil.getFileMap(ids, 1);

            //获取包装附件
            Map<Long, List<FileInfoVo>> packageFileMap = ObsFileUtil.getFileMap(ids, 2);
            for (ContractVo p : list) {
                if (MapUtils.isNotEmpty(cusMap)) {//客户
                    List<Customer> customers = cusMap.getOrDefault(p.getBuyCorporationId(), null);
                    p.setBuyCorporationName(customers == null ? null : customers.get(0).getName());
                }

                //赋值交接单附件
                if (MapUtils.isNotEmpty(fileMap)){
                    List<FileInfoVo> fileInfoVos = fileMap.get(p.getId());
                    p.setFileInfoVos(fileInfoVos);
                }

                //赋值包装附件信息
                if (MapUtils.isNotEmpty(packageFileMap)){
                    List<FileInfoVo>  packageFileInfoVOList= packageFileMap.get(p.getId());
                    p.setPackageFileInfoVOList(packageFileInfoVOList);
                }

            }
        }
        return page;
    }

    @Override
    public ContractVo detail(Long id) {
        Contract Contract = this.getById(id);
        ContractVo result = BeanUtil.toBean(Contract, ContractVo.class);
        return result;
    }

    @DSTransactional
    @Override
    public void add(ContractDto contractDto) {
        setAreaId(contractDto);
        this.save(contractDto);
    }

    @DSTransactional
    @Override
    public void edit(ContractDto contractDto) {
        setAreaId(contractDto);
        this.updateById(contractDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 根据客户ID查询未包装的订单
     *
     * @param customerId
     * @return
     */
    @Override
    public List<ContractVo> getNoPackContractByCustomerId(String customerId) {
        return baseMapper.getNoPackContractByCustomerId(customerId);
    }

    /**
     * 根据合同id集合查询合同列表
     *
     * @param ids
     * @return
     */
    @Override
    public List<Contract> getByIds(List<Long> ids) {
        List<Contract> list = this.list(Wrappers.<Contract>query().lambda().in(Contract::getId, ids));
        return list;
    }

    @Override
    public ContractPdfInfoVo getContractPdfInfo(Long id) {

        if (ObjectUtil.isEmpty(id)) {
            throw new ServiceException("销售合同id不能为空");
        }

        Contract contract = getById(id);
        if (ObjectUtil.isEmpty(contract)) {
            throw new ServiceException("没有找到销售合同");
        }

        ContractPdfInfoVo contractPdfInfoVo = new ContractPdfInfoVo();

        Integer contractType = contract.getContractType();
        if (contractType == null) {
            throw new ServiceException("销售合同类型为空");
        }

        switch (contractType) {
            // 外销合同
            case 1:
                setExportContractInfo(contractPdfInfoVo, contract);
                break;
            // 内销合同
            case 2:
                // todo 内销合同实现逻辑
                break;
            default:
                throw new ServiceException("未知销售合同类型");
        }

        return contractPdfInfoVo;
    }

    /**
     * 查询销售额（合同总金额）
     *
     * @param id (买方公司ID)
     */
    @Override
    public ContractVo getSalesTotal(Long id) {
        return baseMapper.getSalesTotal(id);
    }

    /**
     * 查询指定客户的每月合同总额
     *
     * @param dto
     */
    @Override
    public List<ContractVo> getAmount(CustomerDto dto) {

        return baseMapper.getAmount(dto);
    }

    /**
     * 查询成交单统计（合同）
     *
     * @param query
     * @return
     */
    @Override
    public List<SaleQuotationVo> contractStatistics(QueryWrapper<Object> query) {
        return baseMapper.contractStatistics(query);
    }

    /**
     * 销售趋势（数据看板-产品分析页面）
     *
     * @param productInfoDto
     */
    @Override
    public List<Map<String, Object>> saleTrend(ProductInfoSelectDto productInfoDto) {
        //存放每月的月度销售额与月度销售量的列表
        List<Map<String, Object>> list = new ArrayList<>();

        List<String> monthList = DateUtils.getMonthBetweenDate(productInfoDto.getBeginTime(), productInfoDto.getEndTime());
        //查询月度销售额与月度销售量
        QueryWrapper<ContractProduct> query = Wrappers.query();
        String beginTime = DateUtil.format(productInfoDto.getBeginTime(), "yyyy-MM");
        String endTime = DateUtil.format(productInfoDto.getEndTime(), "yyyy-MM");
        query.ge("DATE_FORMAT(create_time,'%Y-%m')", beginTime);
        query.le("DATE_FORMAT(create_time,'%Y-%m')", endTime);
        query.select("DATE_FORMAT(create_time,'%Y-%m') time,ifnull(SUM(amount),0) contractAmount," +
                "ifnull(SUM(quantity),0) contractQuantity");
        query.groupBy("time");
        List<ContractProduct> contractList = contractProductService.list(query);
        Map<String, List<ContractProduct>> contractMap = contractList.stream().collect(Collectors.groupingBy(ContractProduct::getTime));

        //给每月赋值月度销售额与月度销售量的值
        for (String month : monthList) {
            //存放每月的月度销售额与月度销售量的值
            Map<String, Object> map = new HashMap<>();
            //赋值初使值
            map.put("month", month);
            map.put("contractAmount", new BigDecimal(0));
            map.put("contractQuantity", new BigDecimal(0));

            //赋值
            List<ContractProduct> contracts = contractMap.get(month);
            if (ObjectUtil.isNotEmpty(contracts)) {
                map.put("contractAmount", contracts.get(0).getContractAmount());
                map.put("contractQuantity", contracts.get(0).getContractQuantity());
            }
            list.add(map);
        }

        return list;
    }

    /**
     * 销售统计(数据看板-销售分析页面开发)
     */
    @Override
    public Map<String, Object> salesStatistics(ContractDto dto) {
        //添加查询条件
        QueryWrapper<Contract> query = Wrappers.<Contract>query();
        selectTime(query, dto);
        query.select("sum(amount) amount");

        //查询销售额
        Contract contract = baseMapper.selectOne(query);
        query.clear();
        //查询订单数
        selectTime(query, dto);

        Long count = baseMapper.selectCount(query);

        //查询下单的客户人数
        query.clear();
        selectTime(query, dto);
        query.select("count(DISTINCT(buy_corporation_id)) count");
        Contract contract1 = baseMapper.selectOne(query);

        //存放销售统计数据
        Map<String, Object> map = new HashMap<>();
        //初始化数据
        map.put("amount", new BigDecimal(0));
        map.put("contractCount", count);
        map.put("customerCount", new BigDecimal(0));

        //赋值
        if (ObjectUtil.isNotEmpty(contract)) {
            map.put("amount", contract.getAmount());
        }

        if (ObjectUtil.isNotEmpty(contract1)) {
            map.put("customerCount", contract1.getCount());
        }
        return map;
    }

    /**
     * 国家销售统计(数据看板-销售分析页面开发)
     */
    @Override
    public List<ContractDto> countrySalesStatistics(ContractDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.ge("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getEndTime());
        query.groupBy("cc.buy_country_id");
        query.orderByDesc("amount");
        //查询销售国家统计信息
        List<ContractDto> contractDtoList = baseMapper.countrySalesStatistics(query);
        return contractDtoList;
    }

    /**
     * 客户销售统计(数据看板-销售分析页面开发)
     */
    @Override
    public List<ContractDto> customSalesStatistics(ContractDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.ge("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getEndTime());
        query.groupBy("cc.buy_corporation_id");
        query.orderByDesc("amount");
        //查询销售国家统计信息
        List<ContractDto> contractDtoList = baseMapper.customSalesStatistics(query);
        return contractDtoList;
    }

    /**
     * 业务员销售统计(数据看板-销售分析页面开发)
     */
    @Override
    public List<ContractDto> salesmanSalesStatistics(ContractDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.ge("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(cc.create_time,'%Y-%m')", dto.getEndTime());
        query.groupBy("cc.create_user");
        query.orderByDesc("amount");
        //查询销售国家统计信息
        List<ContractDto> contractDtoList = baseMapper.salesmanSalesStatistics(query);
        return contractDtoList;
    }

    /**
     * 产品销售统计(数据看板-销售分析页面开发)
     */
    @Override
    public List<ContractDto> productSalesStatistics(ContractDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.ge("DATE_FORMAT(cp.create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(cp.create_time,'%Y-%m')", dto.getEndTime());
        query.groupBy("cp.product_id");
        query.orderByDesc("amount");
        //查询销售国家统计信息
        List<ContractDto> contractDtoList = baseMapper.productSalesStatistics(query);
        return contractDtoList;
    }

    /**
     * 产品销售量统计(数据看板-销售分析页面开发)
     */
    @Override
    public List<ContractDto> salesVolumeStatistics(ContractDto dto) {
        QueryWrapper<Object> query = Wrappers.query();
        query.ge("DATE_FORMAT(cp.create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(cp.create_time,'%Y-%m')", dto.getEndTime());
        query.groupBy("cp.product_id");
        query.orderByDesc("quantity");
        //查询销售商品统计信息
        List<ContractDto> contractDtoList = baseMapper.salesVolumeStatistics(query);
        return contractDtoList;
    }

    /**
     * 合同交接单
     */
    @Override
    public void contractHandover(ContractDto dto) {
        ObsFileUtil.editFile(dto.getFileList(),dto.getId(),1);
        ObsFileUtil.editFile(dto.getPackageFileList(),dto.getId(),2);
    }


    /**
     * 赋值外销合同信息
     *
     * @param contractPdfInfoVo param
     * @param contract          合同信息
     */
    private void setExportContractInfo(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {

        SecurityContext context = SecurityContextHolder.getContext();

        // 赋值卖方公司基本信息
        CompletableFuture<Void> setSellCorporationInfoFuture = CompletableFuture.runAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    setSellCorporationInfo(contractPdfInfoVo, contract);
                }, threadPoolExecutor);

        // 赋值买方公司基本信息
        CompletableFuture<Void> setBuyCorporationFuture = CompletableFuture.runAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    setBuyCorporation(contractPdfInfoVo, contract);
                }, threadPoolExecutor);

        // 赋值买卖方公司地址
        CompletableFuture<Void> setAreaFuture = CompletableFuture.runAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    setArea(contractPdfInfoVo, contract);
                }, threadPoolExecutor);

        // 赋值商品列表信息
        CompletableFuture<Void> setProductListFuture = CompletableFuture.runAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    setProductList(contractPdfInfoVo, contract);
                }, threadPoolExecutor);

        // 设置收款账号
        CompletableFuture<Void> setAccountManagementFuture = CompletableFuture.runAsync(
                () -> {
                    SecurityContextHolder.setContext(context);
                    setAccountManagement(contractPdfInfoVo, contract);
                }, threadPoolExecutor);

        // 赋值合同基本信息
        setInfo(contractPdfInfoVo, contract);

        CompletableFuture.allOf(
                setSellCorporationInfoFuture,
                setBuyCorporationFuture,
                setAreaFuture,
                setProductListFuture,
                setAccountManagementFuture
        ).join();

    }

    @Override
    public Page<? extends DocumentaryData> getDocumentaryPage(JSONObject selectData, BaseSelectDto dto, List<Long> excludeBusinessId) {

        ContractDocumentaryBo bo = selectData.toJavaObject(ContractDocumentaryBo.class);

        IWrapper<Contract> wrapper = IWrapper.getWrapper();
        wrapper.like(Contract::getCode, bo.getCode());
        wrapper.eq(Contract::getCreateUser, bo.getUserId());
        wrapper.eq(Contract::getStatus, 30);
        wrapper.notIn(Contract::getId, excludeBusinessId);
        wrapper.orderByDesc(Contract::getId);

        Page<Contract> itemPage = page(dto.getPage(), wrapper);
        Page<ContractDocumentaryVo> page = PageUtils.copyPage(itemPage, ContractDocumentaryVo.class);
        List<ContractDocumentaryVo> records = page.getRecords();

        if (records.size() == 0) {
            return page;
        }

        // 赋值客户名称
        customerService.attributeAssign(records, Contract::getBuyCorporationId, (item, customer) -> {
            item.setCustomerName(customer.getName());
        });

        return page;
    }

    /**
     * 赋值卖方公司基本信息
     */
    private void setSellCorporationInfo(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {
        // 卖方公司信息
        Corporation corporation = corporationService.getById(contract.getSellCorporationId());
        if (corporation == null) {
            throw new ServiceException("卖方公司信息为空");
        }
        contractPdfInfoVo.setSellCorporationNameEn(corporation.getNameEn());
        contractPdfInfoVo.setSellContactName(contract.getSellContactName());
        contractPdfInfoVo.setSellContactNumber(contract.getSellContactNumber());
    }

    /**
     * 赋值买方公司基本信息
     */
    private void setBuyCorporation(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {
        Customer customer = customerService.getById(contract.getBuyCorporationId());
        if (customer == null) {
            throw new ServiceException("买方公司信息为空");
        }
        contractPdfInfoVo.setBuyCorporationName(customer.getName());
        contractPdfInfoVo.setBuyContactName(contract.getBuyContactName());
        contractPdfInfoVo.setBuyContactNumber(contract.getBuyContactNumber());
    }

    /**
     * 赋值买卖公司地址
     */
    private void setArea(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {

        List<Long> areaIdList = Arrays.asList(
                contract.getBuyCountryId(),
                contract.getBuyProvinceId(),
                contract.getBuyCityId(),
                contract.getSellCountryId(),
                contract.getSellProvinceId(),
                contract.getSellCityId()
        );

        Map<Long, String> areaMap = CustomizeAreaUtil.getAreaMapByIds(areaIdList);
        contractPdfInfoVo.setSellCountryName(areaMap.get(contract.getSellCountryId()));
        contractPdfInfoVo.setSellProvinceName(areaMap.get(contract.getSellProvinceId()));
        contractPdfInfoVo.setSellCityName(areaMap.get(contract.getSellCityId()));
        contractPdfInfoVo.setSellDetailedAddress(contract.getSellAddress());
        contractPdfInfoVo.setBuyCountryName(areaMap.get(contract.getBuyCountryId()));
        contractPdfInfoVo.setBuyProvinceName(areaMap.get(contract.getBuyProvinceId()));
        contractPdfInfoVo.setBuyCityName(areaMap.get(contract.getBuyCityId()));
        contractPdfInfoVo.setBuyDetailedAddress(contract.getBuyAddress());
    }

    /**
     * 赋值合同基本信息
     */
    private void setInfo(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {
        contractPdfInfoVo.setContractCode(contract.getCode());
        contractPdfInfoVo.setCreateTimeEn(sdf.format(contract.getCreateTime()));
        contractPdfInfoVo.setTransportRemark(contract.getTransportRemark());
        contractPdfInfoVo.setTradeMethods(contract.getTradeMethods());
        contractPdfInfoVo.setRemark(contract.getRemark());
        contractPdfInfoVo.setCurrency(contract.getCurrency());
        contractPdfInfoVo.setDeliveryTime(contract.getDeliveryTime());
        contractPdfInfoVo.setTransportMethod(contract.getTransportMethod());
        contractPdfInfoVo.setShroffAccountId(contract.getShroffAccountId());
    }

    /**
     * 赋值商品列表信息
     */
    private void setProductList(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {

        contractPdfInfoVo.setTotalAmount(BigDecimal.ZERO);

        List<ContractProduct> list = contractProductService.list(q -> q.eq(ContractProduct::getContractId, contract.getId()));

        if (list.size() == 0) {
            return;
        }

        List<ContractPdfInfoVo.ProductInfo> productInfoList = list.stream().map(item -> {
            contractPdfInfoVo.setTotalAmount(contractPdfInfoVo.getTotalAmount().add(item.getAmount()));

            ContractPdfInfoVo.ProductInfo productIndo = new ContractPdfInfoVo.ProductInfo();
            productIndo.setProductId(item.getProductId());
            productIndo.setProductQuantity(item.getQuantity());
            productIndo.setProductPrice(item.getPrice());
            productIndo.setAmount(item.getAmount());
            productIndo.setProductName(item.getProductName());
            return productIndo;
        }).collect(Collectors.toList());

        contractPdfInfoVo.setProductInfoList(productInfoList);

        // 产品属性
        productInfoService.attributeAssign(productInfoList, ContractPdfInfoVo.ProductInfo::getProductId, (item, product) -> {
            item.setProductUnit(product.getUnit());
        });

    }

    private void setAccountManagement(ContractPdfInfoVo contractPdfInfoVo, Contract contract) {
        Long shroffAccountId = contract.getShroffAccountId();
        AccountManagement accountManagement = accountManagementService.getById(shroffAccountId);
        if (accountManagement == null) {
            return;
        }

        contractPdfInfoVo.setBeneficiaryName(accountManagement.getBeneficiaryName());
        contractPdfInfoVo.setBeneficiaryBank(accountManagement.getBeneficiaryBank());
        contractPdfInfoVo.setBeneficiaryBankAddress(accountManagement.getBeneficiaryBankAddress());
        contractPdfInfoVo.setBeneficiaryAccountNumber(accountManagement.getBeneficiaryAccountNumber());
        contractPdfInfoVo.setSwiftCode(accountManagement.getSwiftCode());
        contractPdfInfoVo.setBeneficiaryAddress(accountManagement.getBeneficiaryAddress());
    }

    // 添加时间的查询条件
    private void selectTime(QueryWrapper<Contract> query, ContractDto dto) {
        query.ge("DATE_FORMAT(create_time,'%Y-%m')", dto.getBeginTime());
        query.le("DATE_FORMAT(create_time,'%Y-%m')", dto.getEndTime());
    }

    private void setAreaId(ContractDto contractDto) {

        SetCustomizeAreaId setCustomizeAreaId1 = new SetCustomizeAreaId() {

            @Override
            public void setProvinceId(Long provinceId) {
                contractDto.setBuyProvinceId(provinceId);
            }

            @Override
            public void setCityId(Long cityId) {
                contractDto.setBuyCityId(cityId);
            }

            @Override
            public Long getCountryId() {
                return contractDto.getBuyCountryId();
            }

            @Override
            public Long getProvinceId() {
                return contractDto.getBuyProvinceId();
            }

            @Override
            public String getProvinceName() {
                return contractDto.getBuyProvinceName();
            }

            @Override
            public Long getCityId() {
                return contractDto.getBuyCityId();
            }

            @Override
            public String getCityName() {
                return contractDto.getBuyCityName();
            }
        };

        SetCustomizeAreaId setCustomizeAreaId2 = new SetCustomizeAreaId() {

            @Override
            public void setProvinceId(Long provinceId) {
                contractDto.setSellProvinceId(provinceId);
            }

            @Override
            public void setCityId(Long cityId) {
                contractDto.setSellCityId(cityId);
            }

            @Override
            public Long getCountryId() {
                return contractDto.getSellCountryId();
            }

            @Override
            public Long getProvinceId() {
                return contractDto.getSellProvinceId();
            }

            @Override
            public String getProvinceName() {
                return contractDto.getSellProvinceName();
            }

            @Override
            public Long getCityId() {
                return contractDto.getSellCityId();
            }

            @Override
            public String getCityName() {
                return contractDto.getSellCityName();
            }
        };

        CustomizeAreaUtil.setAreaId(Arrays.asList(setCustomizeAreaId1, setCustomizeAreaId2));

    }

}
